@@ -9,7 +9,7 @@ from ..utils.comparisons import get_best_match
 _WAITING_TIME_AFTER_ERROR = 0.25
 _MANDATORY_COLUMNS = {'year', 'make', 'model', 'bodytype', 'transmission', 'fueltype', 'trim', 'km'} 
 _COLS_TO_MATCH = ['make', 'model', 'bodytype', 'transmission', 'fueltype', 'trim']
-
+_KMS_CLOSE = 100
 
 class SmartIq_Helper_v2():
     def __init__(self, url_quotation, api_user, api_key, redis_host, redis_port, redis_password):
@@ -173,27 +173,38 @@ class SmartIq_Helper_v2():
         TODO: Remove io from here and move to Camunda
         """
         self.redis_conn.select(1) # db=1 for price cache
+
         l, x1, x2, c0, c1, st = self.redis_conn.hmget('siq-price:' + vehicle_str, ['labels', 'x1', 'x2', 'c0', 'c1', 'strategy'])
-        y1, y2 = self.redis_conn.hmget('siq-price:' + vehicle_str, ['y1', 'y2'])
-        # logging.info(f"a={(x1, y1)}, b={x2, y2}, c0={c0}, c1={c1} ")
-        if not all([x1, x2, c0, c1]) or not int(x1) <= km <= int(x2): # only interpolate, do not extrapolate
-            return None, None
-        
-        retail = int(c0) + float(c1) * int(km)
+        y1, y2 = self.redis_conn.hmget('siq-price:' + vehicle_str, ['y1', 'y2']) 
+
+        logging.info(f"a={(x1, y1)}, b={x2, y2}, c0={c0}, c1={c1} st={st}")
+
+        if all([x1, x2, c0, c1, st]) and int(x1) <= km <= int(x2):
+            retail = int(c0) + float(c1) * int(km)
+            logging.debug(f"price from cache with regression")
+        elif all([x1, st]) and (abs(float(x1) - km) <= _KMS_CLOSE):
+            logging.debug(f"price from cache with single value")
+            retail = int(y1)
+        else:
+            return None,None
+
         io = retail / (1 + int(st)/100)
         return retail, io
 
 
     def cache_price(self, vehicle_str, labels, km, retail, io):
         """ Store price in the cache"""
+        self.redis_conn.select(1)
+
         l, x1, x2, y1, y2 = self.redis_conn.hmget('siq-price:' + vehicle_str, ['labels', 'x1', 'x2', 'y1', 'y2'])
 
         if not x1: # First point
-            self.redis_conn.hmset('siq-price:' + vehicle_str, {'x1': km, 'y1': retail})
+            d = {'labels': labels, 'x1': km, 'y1': retail, 'strategy': round(((retail/io) - 1) * 100)}
+            self.redis_conn.hmset('siq-price:' + vehicle_str, d)
             return
 
         x1, y1 = int(x1), int(y1)
-        if abs(x1 - km) < 100: # to close, ignore
+        if abs(x1 - km) < _KMS_CLOSE: # to close, ignore
             return
         
         if not x2: # Second point
@@ -216,14 +227,14 @@ class SmartIq_Helper_v2():
         c1 = (y1_ - y2_) / (x1_ - x2_)
         c0 = ((y1_ + y2_) - c1 * (x1_ + x2_)) / 2
 
-        d = {'labels': labels, 'x1': x1_, 'x2': x2_, 'y1': y1_, 'y2': y2_
-            , 'c0': round(c0), 'c1': round(c1, 4), 'strategy': round(((retail/io) - 1) * 100)}
-        
-        logging.info(f"caching price for {vehicle_str} as {d}") 
-        self.redis_conn.select(1)
-        self.redis_conn.hmset('siq-price:' + vehicle_str, d)
+        if c1 > 0:
+            logging.warning(f'positive km slope {c1}, removing vehicle {vehicle_str} from cache')
+            self.redis_conn.delete('siq-price:' + vehicle_str)
+        else:
+            d = {'labels': labels, 'x1': x1_, 'x2': x2_, 'y1': y1_, 'y2': y2_
+                , 'c0': round(c0), 'c1': round(c1, 4), 'strategy': round(((retail/io) - 1) * 100)}
+
+            logging.info(f"caching price for {vehicle_str} as {d}") 
+            self.redis_conn.hmset('siq-price:' + vehicle_str, d)
 
         return
-
-
-
